#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import argparse as ap
import logging
import logging.handlers
import os
import sys

from .. import version
from ..pymygdala import LogHandler
from .weblogUtil import Weblog

# Global logger access:
logger = logging.getLogger("thumbnail_cheater")
logger.setLevel(logging.WARN)

#
# using the old quicklook weblog raiding, expand it to do AUDI continuum & cubes too.
#
# The thumbnails don't exactly match their filename, so we might need to modify the name somewhat
# (undo the +/- -> underscore transformation, remove the .sky  portion) to mesh with the manifest
# building utility
#
#
#

_DESCRIPTION = """Cheating Thumbnail Utility, version {}:\n\tFinds, unpacks, and searches a weblog for thumbnail \n\t
images.  Searchings in stage7 (quicklook mode) or stages 10 and 12 (audi mode) \n\t
for images approximating that of the provided file."""


def _make_parser():
    r"""Build a command line and/or argument parser"""

    result = ap.ArgumentParser(description=_DESCRIPTION.format(version), formatter_class=ap.RawTextHelpFormatter)
    result.add_argument(
        "-P", "--profile", action="store", default="", help="profile name to use, e.g. test, production"
    )
    result.add_argument("-v", "--verbose", action="store_true", help="Provide more log feedback.")
    result.add_argument("-q", "--quicklook", action="store_true", help="Search using quicklook assumptions (stage7)")
    result.add_argument("-a", "--audi", action="store_true", help="Search using audi assumptions (stages 10 and 12)")
    result.add_argument("filename", type=str, action="store", help="name of the science product of interest")
    return result


def weblog_raid():
    parser = _make_parser()
    arguments = parser.parse_args()

    # Shamelessly stolen from epilogue with a twist: allow for explict profile setting via the CL
    if "CAPO_PROFILE" not in os.environ and "" == arguments.profile:
        # try to synthesize a profile from our installation root
        profile = os.path.abspath(sys.argv[0]).split(os.path.sep)[-3]
        os.environ["CAPO_PROFILE"] = profile
        print("No CAPO_PROFILE explicitly set, synthesizing {0} by path".format(str(profile)))
    elif "" != arguments.profile:
        os.environ["CAPO_PROFILE"] = arguments.profile

    # integrate the logs with pymygdala's logging system
    handler = LogHandler(profile=os.environ["CAPO_PROFILE"], application="thumbnail_cheater")

    if arguments.verbose:
        handler.setLevel(logging.DEBUG)
        logger.setLevel(logging.DEBUG)
        logger.debug("Enabling more complete feedback.")
    else:
        handler.setLevel(logging.WARN)

    logger.addHandler(handler)

    # A last check: pick a mode
    if (not arguments.audi) and (not arguments.quicklook):
        logger.error("Only two choices: audi or quicklook.  Pick one.")
        exit(-5)

    # Get the contents of our working directory (ingestion's staging area)
    contents = os.listdir(path=os.getcwd())
    logger.debug(f"Searching for a weblog within: {contents}")

    # and find the weblog file among the list (stolen from persistimages): provides the index in contents
    weblog_candidates = [place for place, name in enumerate(contents) if "weblog" in name]

    # We expect a single file to match the pattern:
    if len(weblog_candidates) != 1:
        # So throw an exception when that's not the case.
        logger.error("Cannot isolate the weblog, exiting.")
        exit(-3)

    weblog_path = os.getcwd()
    weblog_filename = contents[weblog_candidates[0]]
    weblog = Weblog(weblog_path, weblog_filename)

    logger.debug(f"hunting in the weblog ({weblog_filename}) for a companion to: {arguments.filename}")

    if arguments.quicklook:
        thumbnail = weblog.find_quicklook_thumbnail_for(arguments.filename)
    else:
        if "cube" in arguments.filename:
            thumbnail = weblog.find_cube_thumbnail_for(arguments.filename)
        else:
            thumbnail = weblog.find_continuum_thumbnail_for(arguments.filename)

    if thumbnail is None:
        if "tt1" in arguments.filename:
            logger.debug(f"Didn't find a thumbnail for {arguments.filename}, as expected")
            exit(0)
        else:
            logger.error(f"Failed to find expected thumbnail for {arguments.filename}")
            exit(-13)

    logger.debug(f"Discovered {thumbnail} for {arguments.filename}.")

    #
    # So, now we have the thumbnail, but we need a variant on it's name copied out into
    # the working directory.  (dump the s10_0 sky iter1.image etc).
    #
    # the thumbnail matching algorithm will need to handle the +/- -> _  that's less easy
    # from this direction.
    #
    desired_thumbnail = os.path.basename(thumbnail)
    desired_thumbnail = desired_thumbnail.replace(".s12_0", "")
    desired_thumbnail = desired_thumbnail.replace(".s10_0", "")
    desired_thumbnail = desired_thumbnail.replace(".sky", "")
    # AUDI images drop the .iter1.image bit, but vlass qls don't.
    if not arguments.quicklook:
        desired_thumbnail = desired_thumbnail.replace(".iter1.image", "")

    logger.debug(f"Copying the thumbnail to {desired_thumbnail}")
    os.link(thumbnail, desired_thumbnail)
